;this is a test if we can produce something on the Creativision.
;The Creativision has a 6502 processor (same as the Atari 2600),
;a TMS9928 video chip mapped to addresses $2000 (data read), $2001 (status read), $3000 (data write) and $3001 (status write)
;and a SN76489 sound chip mapped to address $1002 (write).
;The remainder is unimportant at the moment.
; 
	processor 6502
;	include vcs.h ;we don't need that here since we're doing a different system this time.

; The vhs.h file here is a bit special. In fact, I didn't take this one
; anywhere from the list, but took the equates from a listing Distella
; generated...

;	org $7800	;we have to add 2 more K in order to have the cart image accepted
;	.byte #$00	;dummy so we have something in that range
;	org $8000	;16K Creativision carts usually go from $8000-$BFFF.

       
; Here come some equates specific to the Creativision:

VDP_Status_Write = $3001
VDP_Data_Write = $3000
VDP_Data_Read = $2000
VDP_Status_Red = $2001
Sound_Write = $1002
Sound_Status = $1003

BIOS_Sound_Write = $FE77 ;this is the address of a BIOS subroutine which writes a byte to the sound chip, then waits until it's not busy anymore (?)
BIOS_Interrupt_Return = $FF52 ;this is another BIOS routine which returns from an interrupt
BIOS_Interrupt_Handler = $FF3F ;this is the BIOS's internal interrupt handler; it saves all the registers, stores the VDP status byte in RAM and then jumps to the
				;cartridge-internal interrupt handler through vector $BFEA
BIOS_VDP_Regwrite = $FE1F
BIOS_VDP_Regsetup = $F84B

;The following are variables kept in RAM. Possible RAM locations are $00 through $3FF.

;At this time, I haven't got any men in here. I'll change that sometime...
;For simplicity, we keep the ones we had on the Atari 2600 for the music there.

Temp       = $82
SoundCount = $83
DecayCount = $84
SoundBytePointer = $85 ;actually, $85 through $86
NoteBeingPlayed = $87 ;$87 for SG 0 and $88 for SG 1 and $89 for SG 2
GraphicsPointer = $8A
;Here comes the start part. Since at the moment, we only output sound, no video, we only need to initialize the sound pointers.

	org $A000
	include PatternGeneratorTable.asm


Start

	SEI	;Disable interrupts until everything is set up
	LDX #$FF
	LDA #00
ClearMemLoop:
	STA 0,X
	STA 0100,X
	STA 0200,X
	STA 0300,X
	DEX
	BNE ClearMemLoop

;Initialize Sound Pointer
	LDA #$BD
	STA SoundBytePointer+1
	LDA #$00
	STA SoundBytePointer
;Initialize DecayCount
	LDA #$01
	STA DecayCount
;And we're done with initializing... clear interrupts and get caught in an endless loop since all sound operations are interrupt driven ;-)
;For test purposes, set VDC to a different background color
	LDX #07
	LDA #$88
;	JSR BIOS_VDP_Regwrite
	LDX #01
	LDA #$E0
;	JSR BIOS_VDP_Regsetup
;	JSR BIOS_VDP_Regwrite ;rewrite VDP Register 1 just in case

;Now we initialize the VDP so that we're also able to see something...
	LDA #$00		;First write Pattern Generator Table into VRAM at $0000
	STA VDP_Status_Write	;LSB of the write address
	LDA #$40
	STA VDP_Status_Write	;MSB and command of write address - set write address to $0000

	LDA #$00
	STA GraphicsPointer
	LDA #$A0
	STA GraphicsPointer+1

	LDX #$18
	LDY #$00

PatternCopy	;copy pattern generator table
	LDA (GraphicsPointer),Y
	;TYA
	STA VDP_Data_Write
	INY
	BNE PatternCopy
	INC GraphicsPointer+1
	DEX
	BNE PatternCopy

;Write Pattern Name Table
	
	LDX #$00
	LDY #$03	;3 x 256 bytes
VDP_Write_Outer_Loop	
	STX VDP_Data_Write
	INX
	BNE VDP_Write_Outer_Loop
	DEY
	BNE VDP_Write_Outer_Loop

;Write Color Table
	LDA #$00		
	STA VDP_Status_Write	;LSB of the write address
	LDA #$60
	STA VDP_Status_Write	;MSB and command of write address

	LDA #$00
	STA GraphicsPointer
	LDA #$80
	STA GraphicsPointer+1

	
	LDX #$18
	LDY #$00

ColorCopy		;copy color table
	LDA (GraphicsPointer),Y
	;TYA
	STA VDP_Data_Write
	INY
	BNE ColorCopy
	INC GraphicsPointer+1
	DEX
	BNE ColorCopy
EndGraphicsLoop
	CLI	;enable interrupts now
EndlessLoop1
	LDY #$FF
EndlessLoop
	TYA
	LDX #07
;	JSR BIOS_VDP_Regwrite
	DEY
	BNE EndlessLoop
	;JSR Internal_Interrupt_Handler
	JMP EndlessLoop1

Internal_Interrupt_Handler:
	DEC DecayCount
	LDA DecayCount
	LDX #07
	;JSR BIOS_VDP_Regwrite
;Now process loudness for both sound generators
	LDX #2
SoundGeneratorLoop
	LDA #$F0	;turn noise generator up fully
	LDA NoteBeingPlayed,X
	CMP #$FF
	BEQ NoNotePlayed
	LDA DecayCount
	EOR #$FF
	AND #$9F
	STA Temp
	TXA
	ROL	;move PSG number in bits 5 and 6
	ROL
	ROL
	ROL
	ROL
	CLC
	ADC Temp
	JSR BIOS_Sound_Write
NoNotePlayed
	DEX
	CPX #$FF
	BNE SoundGeneratorLoop
HandleNewNotes	
	LDA DecayCount
	BNE EndInterrupt

	LDA #$0F
	STA DecayCount
ReadNewNotes1
	LDX #2
ReadNewNotes
	STX Temp
	LDY #0
	LDA (SoundBytePointer),Y	;fetch note number to be played
	STA NoteBeingPlayed,X
	CMP #$FE
	BEQ ReturnToStart
	CMP #$FF
	BEQ PlayNoNote
	ROL
	TAX				;x now contains index into frequency table for that note
	INX
	INX
	LDA Temp			;load PSG number
	ROL
	ROL
	ROL
	ROL
	ROL				;now in bits 5 and 6
	CLC
	ORA #$80			;set MSB to signal Frequency setting
	ADC FrequencyTable,X		;add low byte of desired note
	JSR BIOS_Sound_Write		;write first byte to PSG
	LDA FrequencyTable+1,X		;load high byte of desired note (no processing needed)
	JSR BIOS_Sound_Write		;write it out to sound generator
PlayNoNote				;this is where we continue if this note doesn't get played
	INC SoundBytePointer
	BNE Noincrease
	INC SoundBytePointer+1
Noincrease
	LDX Temp
	DEX
	CPX #$FF
	BNE ReadNewNotes
EndInterrupt
	JMP BIOS_Interrupt_Return	;end interrupt handling
	RTI
	;RTS

ReturnToStart				;init Sound Pointer again
	LDA #$BD
	STA SoundBytePointer+1
	LDA #$00
	STA SoundBytePointer
	JMP ReadNewNotes1


	org $BC00	;here we'll store the frequency for each possible note
FrequencyTable
	.word $3F0F  ;lowest note = C0; 00 111111 0000 1111 = 1023
	.word $3C06  ;C#0; 00 111100 0000 0110 = 966
	.word $380F ;D0;	00 111000 0000 1111 = 911
	.word $350C; D#0; 00 110101 0000 1100 = 860
	.word $320C; E0; 00 110010 0000 1100 = 812
	.word $2F0E; F0; 00 101111 0000 1110 = 766
	.word $2D03; F#0;00 101101 0000 0011 = 723
	.word $2A8B; G0;00 101010 0000 1011= 683
	.word $2804; G#0;00 101000 0000 0100 = 644
	.word $2600; A0;00 100110 0000 0000 = 608
	.word $230E; A#0;00 100011 0000 1110 = 574
	.word $210E; H0;00 100001 0000 1110 = 542
	.word $2000; C1;00 100000 0000 0000 = 512
	.word $1E03; C#1;00 011110 0000 0011 = 483
	.word $1C07; D1;00 011100 0000 0111 = 455
	.word $1A0E; D#1;00 011010 0000 1110 = 430
	.word $1906; E1;00 011001 0000 0110 = 406
	.word $170F; F1;00 010111 0000 1111 = 383
	.word $160A; G1;00 010110 0000 1010 = 362
	.word $1505; F#1;00 010101 0000 0101 = 341
	.word $1402; G#1;00 010100 0000 0010 = 322
	.word $1300; A1;00 010011 0000 0000 = 304
	.word $110F; A#1;00 010001 0000 1111 = 287
	.word $100F; H1;00 010000 0000 1111 = 271
	.word $1000; C2; = 256
	.word $0F01; C#2 = 241
	.word $0E04; D2  = 228
	.word $0D07; D#2  = 215
	.word $0C0B; E2  = 203
	.word $0C00; F2  = 192
	.word $0B05; F#2  = 181
	.word $0A0B; G2  = 171
	.word $0A01; G#2  = 161
	.word $0908; A2  = 152
	.word $0900; A#2  = 144
	.word $0807; H2 = 135

	org $BD00	;here we'll store the melody data; 255 means pause, 254 means back to start
	.byte 5+0, 5+12+4, 5+12+0
	.byte 5+7, 5+12+0, 5+12-5
	.byte 5+4, 5+12+0, 5+12-5
	.byte 5+7, 5+12+0, 5+12-5
	.byte 5+0, 5+12+4, 5+12+0
	.byte 5+7, 5+12+0, 5+12-5
	.byte 5+4, 5+12+0, 5+12-5
	.byte 5+7, 5+12+0, 5+12-5

	.byte 5+2, 5+12+5, 5+12+2
	.byte 5+7, 5+12+5, 5+12+2
	.byte 5+5, 5+12+4, 5+12
	.byte 5+7, 5+12+4, 5+12
	.byte 5+2, 5+12+2, 5+12-1
	.byte 5+7, 255, 255 
	.byte 5+5, 255, 255
	.byte 5+7, 255, 255

	.byte 5+2, 5+12+5, 5+12+2
	.byte 5+7, 5+12+5, 5+12+2
	.byte 5+5, 5+12+4, 5+12
	.byte 5+7, 5+12+4, 5+12
	.byte 5+2, 5+12+2, 5+12-1
	.byte 5+7, 255, 255
	.byte 5+5, 5+12+9, 5+12+5
	.byte 5+7, 255, 255

	.byte 5+2, 5+12+7, 5+12+4
	.byte 5+7, 5+12+5, 5+12+2
	.byte 5+5, 5+12+4, 5+12
	.byte 5+7, 5+12+2, 5+12-1
	.byte 5+0, 5+12+0, 5+12-5
	.byte 5+7, 255, 255
	.byte 5+4, 255, 255
	.byte 5+7, 255, 255


	.byte 5+0, 5+12+7, 255
	.byte 255, 255, 255
	.byte 255, 5+12+4, 255
	.byte 255, 5+12+0, 255
	.byte 5-3, 5+12+12, 255
	.byte 255, 255, 255
	.byte 255, 5+12+11, 255
	.byte 255, 5+12+9, 255

	.byte 5-5, 5+12+7, 255
	.byte 255, 255, 255
	.byte 255, 5+12+12, 255
	.byte 255, 5+12+4, 255
	.byte 5-1, 5+12+7, 255
	.byte 255, 5+12+2, 255
	.byte 5-3, 255, 255
	.byte 5-1, 255, 255


	.byte 5+0, 255, 255
	.byte 5+7, 5+12+7, 5+12+4
	.byte 5+4, 5+12+7, 5+12+4
	.byte 5+7, 5+12+7, 5+12+4
	.byte 5+0, 5+12+7, 5+12+4
	.byte 5+7, 5+12+4, 5+12-0
	.byte 5+4, 5+12+2, 5+12-5
	.byte 5+7, 5+12+0, 255

	.byte 5+0, 255, 255
	.byte 5+5, 5+12+12, 5+12+9
	.byte 5-3, 5+12+12, 5+12+9
	.byte 5+5, 5+12+12, 5+12+9
	.byte 5+0, 5+12+14, 5+12+11
	.byte 5+5, 5+12+12, 5+12+9
	.byte 5-3, 5+12+11, 5+12+7
	.byte 5+5, 5+12+9, 5+12+5

	.byte 5+0, 5+12+7, 5+12+4
	.byte 5+7, 5+12+4, 5+12+0
	.byte 5+4, 255, 255
	.byte 5+7, 5+12+4, 5+12+0
	.byte 5+0, 5+12+12, 5+12+4
	.byte 5+7, 255, 255
	.byte 5+4, 5+12+4, 5+12-0
	.byte 5+7, 255, 255

	.byte 5-1, 5+12+7, 5+12+2
	.byte 5+2, 5+12+2, 5+12-5
	.byte 5-5, 255, 255
	.byte 5+2, 255, 255
	.byte 5-3, 255, 255
	.byte 5+2, 255, 255
	.byte 5-1, 255, 255
	.byte 5+2, 255, 255

	.byte 5+0, 255, 255
	.byte 5+7, 5+12+4, 5+12
	.byte 5+4, 5+12+4, 5+12
	.byte 5+7, 5+12+5, 5+12+2
	.byte 5+0, 5+12+7, 5+12+4
	.byte 5+7, 5+12+9, 5+12+5
	.byte 5+4, 5+12+11, 5+12+7
	.byte 5+7, 5+12+9, 5+12+5

	.byte 5+0, 255, 255
	.byte 5+5, 255, 255
	.byte 5-3, 255, 255
	.byte 5+5, 255, 255
	.byte 5+0, 255, 255
	.byte 5+5, 255, 255
	.byte 5-3, 255, 255
	.byte 5+5, 255, 255

	.byte 5+0, 5+12+5, 5+12
	.byte 5+5, 5+12+5, 5+12
	.byte 5-3, 5+12+7, 5+12
	.byte 5+5, 5+12+9, 5+12+5
	.byte 5+0, 255, 255
	.byte 5+5, 5+12+11, 5+12+7
	.byte 5-3, 5+12+12, 5+12+9
	.byte 5+5, 5+12+11, 5+12+7

	.byte 5-1, 255, 255
	.byte 5+2, 255, 255
	.byte 5-5, 255, 255
	.byte 5+2, 255, 255
	.byte 5-3, 255, 255
	.byte 5+2, 255, 255
	.byte 5-1, 5+12+7, 255
	.byte 5+2, 255, 255

	.byte 5+0, 5+12+12, 5+12+4
	.byte 5+7, 255, 255
	.byte 5+4, 255, 255
	.byte 5+7, 255, 255
	.byte 5+0, 5+12+11, 5+12+4
	.byte 5+7, 255, 255
	.byte 5+4, 5+12+9, 5+12+4
	.byte 5+7, 5+12+7, 5+12+4

	.byte 5+0, 5+12+11, 5+12+7
	.byte 5+5, 255, 255
	.byte 5-3, 5+12+9, 5+12+5
	.byte 5+5, 255, 255
	.byte 5+0, 255, 255
	.byte 5+5, 255, 255
	.byte 5-3, 5+12+9, 5+12+5
	.byte 5+5, 255, 255

	.byte 5-1, 5+12+7, 5+12+5
	.byte 5+2, 255, 255
	.byte 5-5, 5+24+2, 5+12+7
	.byte 5+2, 255, 255
	.byte 5-3, 5+12+12, 5+12+5
	.byte 5+2, 255, 255
	.byte 5-1, 5+12+11, 5+12+5
	.byte 5+2, 255, 255

	.byte 5+0, 5+12+12, 5+12+4
	.byte 5+4, 255, 255
	.byte 5-1, 255, 255
	.byte 5+4, 255, 255
	.byte 5-3, 255, 255
	.byte 5+4, 255, 255
	.byte 5-5, 255, 255
	.byte 5+4, 255, 255


	.byte 5+0, 5+24, 5+12+4
	.byte 5+4, 5+24, 5+12+4
	.byte 5-5, 5+24+2, 5+12+5
	.byte 5+4, 5+24+4, 5+12+7
	.byte 5+0, 5+24, 5+12+4
	.byte 5+4, 5+24+4, 5+12+7
	.byte 5+2, 5+24+2, 5+12+5
	.byte 5-5, 5+12+7, 255

	.byte 5+0, 5+24, 5+12+4
	.byte 5+4, 5+24, 5+12+4
	.byte 5-5, 5+24+2, 5+12+5
	.byte 5+4, 5+24+4, 5+12+7
	.byte 5+0, 5+24, 5+12+4
	.byte 5+4, 255, 255
	.byte 5+2, 5+24-1, 5+12+2
	.byte 5-5, 5+12+7, 255

	.byte 5+0, 5+24, 5+12+4
	.byte 5+4, 5+24, 5+12+4
	.byte 5-5, 5+24+2, 5+12+5
	.byte 5+4, 5+24+4, 5+12+7
	.byte 5+0, 5+24+5, 5+12+9
	.byte 5+5, 5+24+4, 5+12+7
	.byte 5-3, 5+24+2, 5+12+5
	.byte 5+5, 5+12+12, 5+12+4

	.byte 5-1, 5+24-1, 5+12+2
	.byte 5+2, 5+12+7, 5+12-1
	.byte 5-5, 5+24-3, 5+12
	.byte 5+2, 5+24-1, 5+12+2
	.byte 5+0, 5+24, 5+12+4
	.byte 5+4, 255, 255
	.byte 5-5, 5+24, 5+12+4
	.byte 5+4, 255, 255

	.byte 5+0, 5+12+9, 5+12+5
	.byte 5+5, 5+12+11, 5+12+7
	.byte 5-3, 5+12+9, 5+12+5
	.byte 5+5, 5+12+7, 5+12+4
	.byte 5+0, 5+12+9, 5+12+5
	.byte 5+5, 5+12+11, 5+12+7
	.byte 5-3, 5+12+12, 5+12+9
	.byte 5+5, 5+12+9, 5+12+5

	.byte 5+0, 5+12+7, 5+12+4
	.byte 5+4, 5+12+9, 5+12+5
	.byte 5-5, 5+12+7, 5+12+4
	.byte 5+4, 5+12+5, 5+12+2
	.byte 5+0, 5+12+4, 5+12
	.byte 5+4, 255, 255
	.byte 5-5, 5+12+7, 5+12+4
	.byte 5+4, 255, 255

	.byte 5+0, 5+12+9, 5+12+5
	.byte 5+5, 5+12+11, 5+12+7
	.byte 5-3, 5+12+9, 5+12+5
	.byte 5+5, 5+12+7, 5+12+4
	.byte 5+0, 5+12+9, 5+12+5
	.byte 5+5, 5+12+11, 5+12+7
	.byte 5-3, 5+12+12, 5+12+9
	.byte 5+5, 5+12+9, 5+12+5

	.byte 5-1, 5+12+7, 5+12+2
	.byte 5+2, 5+12+12, 5+12+7
	.byte 5-5, 5+12+11, 5+12+5
	.byte 5+2, 5+12+14, 5+12+7
	.byte 5+0, 5+12+12, 5+12+4
	.byte 5+4, 255, 255
	.byte 5-5, 5+12+12, 5+12+4
	.byte 5+4, 255, 255

	.byte 254





	org $BFEA	;this is the address where the BIOS' own interrupt handler expects a vector to the cartridge's own interrupt routine
	.word Internal_Interrupt_Handler

	org $BFF0	;this is the table for initializing the VDP on startup which gets read by the BIOS on startup
	;.byte $00	;VDP Reg. 0 -> External Video off, M2=0 -> Graphics Mode
	;.byte $E2	;VDP Reg. 1 -> 16K Mode; Screen active; Generate Interrupts on; M1=0; M3=0; Sprites = 16x16 unmagnified
	;.byte $04	;VDP Reg.2 -> Pattern Name Table = 04 * $400 = $1000
	;.byte $4E	;VDP Reg.3 -> Color Table Adress = 00 01001110 000000 = $1380
	;.byte $00	;VDP Reg.4 -> Pattern Generator Table = $0000
	;.byte $26	;VDP Reg.5 -> Sprite Attribute Table = 0 00100110 0000000 = $1300
	;.byte $01	;VDP Reg.6 -> Sprite Generator Table = 00001 00000000000 = $0800
	;.byte $01	;VDP Reg.7 -> TC=0; BD=1 -> Text Color = 0 (Transparent); Backdrop = 1 (Black)
	
	.byte $02	;VDP Reg. 0 -> External Video off, M2=1 -> Bitmap mode
	.byte $E2	;VDP Reg. 1 -> 16K Mode; Screen active; Generate Interrupts on; M1=0; M3=0; Sprites = 16x16 unmagnified
	.byte $06	;VDP Reg.2 -> Pattern Name Table = 06 * $400 = $1800
	.byte $FF	;VDP Reg.3 -> Color Table Adress = $2000 (Bits 0-6 are an and mask, so they were set to 1 here)
	.byte $03	;VDP Reg.4 -> Pattern Generator Table = $0000; Bits 0-1 are an AND mask
	.byte $36	;VDP Reg.5 -> Sprite Attribute Table = 0 00110110 0000000 = $1B00
	.byte $07	;VDP Reg.6 -> Sprite Generator Table = 00111 00000000000 = $3800
	.byte $01	;VDP Reg.7 -> TC=0; BD=1 -> Text Color = 0 (Transparent); Backdrop = 1 (Black)

;This gives the following VDP memory mapping:
;	$0000-$17FF	Pattern generator table
;	$1800-$1AFF	Pattern name table
;	$1B00-$1B7F	Sprite attribute table
;	$2000-$37FF	Color table
;	$3800-$3FFF	Sprite Generator Table

	org $BFFC
	.word Start
	.word BIOS_Interrupt_Handler
;	.word Internal_Interrupt_Handler
;	org $C7FF	;we do another 2K to fill the 18K cart. These are actually loaded at $7800-$7FFF, but it doesn't matter since they're empty and not used.
;	.byte $00

	org $C000
	include	ColorTable.asm	;Colortable is linked in at $C000, but actually is read to $8000
	org $DFFF
	.byte $00